package test.wildfly;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.annotation.PostConstruct;
import javax.annotation.Resource;
import javax.ejb.EJB;
import javax.inject.Inject;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet(name = "TestServlet", urlPatterns = {"/TestServlet"})
public class TestServlet extends HttpServlet {

    //Scenario 1: basic @EJB
    //Works with Glassfish, not with Wildfly
//    @EJB
//    private TestLocal bean;
    
    //Scenario 2: @EJB with lookup
    //Does not work with Wildfly, not tested on Glassfish
//    @EJB(lookup = "java:app/test-svc-ejb/TestBean!test.wildfly.TestLocal")
//    private TestLocal bean;
    
    //Scenario 3: Manual JNDI lookup
    //Works on Wildfly
//    private TestLocal bean;
//    @PostConstruct
//    private void postConstruct() {
//        try {
//            bean = (TestLocal) InitialContext.doLookup("java:app/test-svc-ejb/TestBean!test.wildfly.TestLocal");
//        } catch (NamingException ex) {
//            Logger.getLogger(TestServlet.class.getName()).log(Level.SEVERE, null, ex);
//        }
//    }
    
    //Scenario 4: using @Inject
    //Works with Wildfly (I'm suprised!)
    @Inject
    private TestLocal bean;
    
    //Scenario 5: using @Resource
    //Does not work with Wildfly
//    @Resource(lookup = "java:app/test-svc-ejb/TestBean!test.wildfly.TestLocal")
//    private TestLocal bean;



    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        response.setContentType("text/html;charset=UTF-8");
        try (PrintWriter out = response.getWriter()) {
            /* TODO output your page here. You may use following sample code. */
            out.println("<!DOCTYPE html>");
            out.println("<html>");
            out.println("<head>");
            out.println("<title>Servlet TestServlet</title>");
            out.println("</head>");
            out.println("<body>");
            out.println("<h1>Servlet TestServlet at " + request.getContextPath() + "</h1>");
            out.println("<div>" + bean.sayHello() + "</div>");
            out.println("</body>");
            out.println("</html>");
        }
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /**
     * Handles the HTTP <code>GET</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Handles the HTTP <code>POST</code> method.
     *
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {
        processRequest(request, response);
    }

    /**
     * Returns a short description of the servlet.
     *
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
